@@ -12,8 +12,7 @@ import sys
 import time
 import warnings
 import weakref
-from collections import MutableSequence, namedtuple
-from functools import total_ordering
+from collections import namedtuple
 from math import ceil
 from pathlib import Path
 from time import gmtime
@@ -22,6 +21,8 @@ from urllib.parse import quote
 from async_timeout import timeout
 
 from . import hdrs
+from .abc import AbstractCookieJar
+
 
 try:
     from asyncio import ensure_future
@@ -39,7 +40,7 @@ else:
 
 
 __all__ = ('BasicAuth', 'create_future', 'parse_mimetype',
-           'Timeout', 'ensure_future', 'noop')
+           'Timeout', 'ensure_future', 'noop', 'DummyCookieJar')
 
 
 sentinel = object()
@@ -53,33 +54,51 @@ SEPARATORS = {'(', ')', '<', '>', '@', ',', ';', ':', '\\', '"', '/', '[', ']',
 TOKEN = CHAR ^ CTL ^ SEPARATORS
 
 
-if sys.version_info < (3, 5):
-    noop = tuple
+class _CoroGuard:
+    __slots__ = ('_coro', '_msg', '_awaited')
 
-    coroutines = asyncio.coroutines
-    old_debug = coroutines._DEBUG
-    coroutines._DEBUG = False
+    def __init__(self, coro, msg):
+        self._coro = coro
+        self._msg = msg
+        self._awaited = False
 
-    @asyncio.coroutine
-    def deprecated_noop(message):
-        warnings.warn(message, DeprecationWarning, stacklevel=3)
+    def __iter__(self):
+        self._awaited = True
+        return self._coro.__iter__()
 
-    coroutines._DEBUG = old_debug
+    if PY_35:
+        def __await__(self):
+            self._awaited = True
+            return (yield from self._coro)
 
-else:
-    coroutines = asyncio.coroutines
-    old_debug = coroutines._DEBUG
-    coroutines._DEBUG = False
+    def __del__(self):
+        self._coro = None
+        if not self._awaited:
+            warnings.warn(self._msg, DeprecationWarning)
 
-    @asyncio.coroutine
-    def noop(*args, **kwargs):
-        return
 
-    @asyncio.coroutine
-    def deprecated_noop(message):
-        warnings.warn(message, DeprecationWarning, stacklevel=3)
+coroutines = asyncio.coroutines
+old_debug = coroutines._DEBUG
+coroutines._DEBUG = False
 
-    coroutines._DEBUG = old_debug
+
+@asyncio.coroutine
+def noop(*args, **kwargs):
+    return
+
+
+def deprecated_noop(message):
+    return _CoroGuard(noop(), message)
+
+
+coroutines._DEBUG = old_debug
+
+
+try:
+    from asyncio import isfuture
+except ImportError:
+    def isfuture(fut):
+        return isinstance(fut, asyncio.Future)
 
 
 class BasicAuth(namedtuple('BasicAuth', ['login', 'password', 'encoding'])):
@@ -134,12 +153,24 @@ if PY_352:
     def create_future(loop):
         return loop.create_future()
 else:
-    def create_future(loop):
+    def create_future(loop):  # pragma: no cover
         """Compatibility wrapper for the loop.create_future() call introduced in
         3.5.2."""
         return asyncio.Future(loop=loop)
 
 
+def current_task(loop=None):
+    if loop is None:
+        loop = asyncio.get_event_loop()
+
+    task = asyncio.Task.current_task(loop=loop)
+    if task is None:
+        if hasattr(loop, 'current_task'):
+            task = loop.current_task()
+
+    return task
+
+
 def parse_mimetype(mimetype):
     """Parses a MIME type into its components.
 
@@ -227,8 +258,7 @@ class AccessLogger:
         %P  The process ID of the child that serviced the request
         %r  First line of request
         %s  Response status code
-        %b  Size of response in bytes, excluding HTTP headers
-        %O  Bytes sent, including headers
+        %b  Size of response in bytes, including HTTP headers
         %T  Time taken to serve the request, in seconds
         %Tf Time taken to serve the request, in seconds with floating fraction
             in .06f format
@@ -245,7 +275,6 @@ class AccessLogger:
         'r': 'first_request_line',
         's': 'response_status',
         'b': 'response_size',
-        'O': 'bytes_sent',
         'T': 'request_time',
         'Tf': 'request_time_frac',
         'D': 'request_time_micro',
@@ -488,57 +517,6 @@ def is_ip_address(host):
                         .format(host, type(host)))
 
 
-@total_ordering
-class FrozenList(MutableSequence):
-
-    __slots__ = ('_frozen', '_items')
-
-    def __init__(self, items=None):
-        self._frozen = False
-        if items is not None:
-            items = list(items)
-        else:
-            items = []
-        self._items = items
-
-    def freeze(self):
-        self._frozen = True
-        self._items = tuple(self._items)
-
-    def __getitem__(self, index):
-        return self._items[index]
-
-    def __setitem__(self, index, value):
-        if self._frozen:
-            raise RuntimeError("Cannot modify frozen list.")
-        self._items[index] = value
-
-    def __delitem__(self, index):
-        if self._frozen:
-            raise RuntimeError("Cannot modify frozen list.")
-        del self._items[index]
-
-    def __len__(self):
-        return self._items.__len__()
-
-    def __iter__(self):
-        return self._items.__iter__()
-
-    def __reversed__(self):
-        return self._items.__reversed__()
-
-    def __eq__(self, other):
-        return list(self) == other
-
-    def __le__(self, other):
-        return list(self) <= other
-
-    def insert(self, pos, item):
-        if self._frozen:
-            raise RuntimeError("Cannot modify frozen list.")
-        self._items.insert(pos, item)
-
-
 class TimeService:
 
     def __init__(self, loop, *, interval=1.0):
@@ -597,6 +575,10 @@ class TimeService:
     def loop_time(self):
         return self._loop_time
 
+    @property
+    def interval(self):
+        return self._interval
+
 
 def _weakref_handle(info):
     ref, name = info
@@ -643,8 +625,11 @@ class TimeoutHandle:
             return self._loop.call_at(at, self.__call__)
 
     def timer(self):
-        timer = TimerContext(self._loop)
-        self.register(timer.timeout)
+        if self._timeout is not None and self._timeout > 0:
+            timer = TimerContext(self._loop)
+            self.register(timer.timeout)
+        else:
+            timer = TimerNoop()
         return timer
 
     def __call__(self):
@@ -675,7 +660,8 @@ class TimerContext:
         self._cancelled = False
 
     def __enter__(self):
-        task = asyncio.Task.current_task(loop=self._loop)
+        task = current_task(loop=self._loop)
+
         if task is None:
             raise RuntimeError('Timeout context manager should be used '
                                'inside a task')
@@ -689,21 +675,14 @@ class TimerContext:
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         if self._tasks:
-            task = self._tasks.pop()
-        else:
-            task = None
+            self._tasks.pop()
 
         if exc_type is asyncio.CancelledError and self._cancelled:
-            for task in self._tasks:
-                task.cancel()
             raise asyncio.TimeoutError from None
 
-        if exc_type is None and self._cancelled and task is not None:
-            task.cancel()
-
     def timeout(self):
         if not self._cancelled:
-            for task in self._tasks:
+            for task in set(self._tasks):
                 task.cancel()
 
             self._cancelled = True
@@ -713,7 +692,7 @@ class CeilTimeout(Timeout):
 
     def __enter__(self):
         if self._timeout is not None:
-            self._task = asyncio.Task.current_task(loop=self._loop)
+            self._task = current_task(loop=self._loop)
             if self._task is None:
                 raise RuntimeError(
                     'Timeout context manager should be used inside a task')
@@ -761,3 +740,30 @@ class HeadersMixin:
             return None
         else:
             return int(l)
+
+
+class DummyCookieJar(AbstractCookieJar):
+    """Implements a dummy cookie storage.
+
+    It can be used with the ClientSession when no cookie processing is needed.
+
+    """
+
+    def __init__(self, *, loop=None):
+        super().__init__(loop=loop)
+
+    def __iter__(self):
+        while False:
+            yield None
+
+    def __len__(self):
+        return 0
+
+    def clear(self):
+        pass
+
+    def update_cookies(self, cookies, response_url=None):
+        pass
+
+    def filter_cookies(self, request_url):
+        return None
